@@ -1,6 +1,8 @@
 import torch
 from torch import nn
 import torch.nn.functional as F
+from einops import rearrange
+
 
 __all__ = ["DiceLoss", "DiceCE"]
 
@@ -11,7 +13,7 @@ MULTICLASS_MODE = "multiclass"
 class DiceCE(nn.Module):
     def __init__(self, w_dice=0.5, w_ce=0.5, cls_weight=None):
         """
-
+        Dice Loss + Cross Entropy Loss
         Args:
             w_dice: weight of Dice Loss
             w_ce: weight of CrossEntropy loss
@@ -33,7 +35,7 @@ class DiceCE(nn.Module):
 class DiceLoss(nn.Module):
     def __init__(
         self,
-        mode="multiclass",  # binary
+        mode="multiclass",  # binary, multiclass
         log_loss=False,
         from_logits=True,
     ):
@@ -53,14 +55,13 @@ class DiceLoss(nn.Module):
 
     def forward(self, y_pred, y_true):
         """
-        :param y_pred: (N, C, H, W)
-        :param y_true: (N, H, W)
+        :param y_pred: (N, C, *)
+        :param y_true: (N, *)
         :return: scalar
         """
         assert y_true.size(0) == y_pred.size(0)
-        bs = y_true.size(0)
         num_classes = y_pred.size(1)
-        dims = (0, 2) # # (N, -1, H*W)
+        dims = (0, 2)  # (N, C, HW)
 
         if self.from_logits:
             # get [0..1] class probabilities
@@ -70,20 +71,18 @@ class DiceLoss(nn.Module):
                 y_pred = torch.sigmoid(y_pred)
 
         if self.mode == BINARY_MODE:
-            y_true = y_true.view(bs, 1, -1)
-            y_pred = y_pred.view(bs, 1, -1)
+            y_true = rearrange(y_true, "N ... -> N 1 (...)")
+            y_pred = rearrange(y_pred, "N 1 ... -> N 1 (...)")
         elif self.mode == MULTICLASS_MODE:
-            y_true = y_true.view(bs, -1)
-            y_pred = y_pred.view(bs, num_classes, -1)
-
-            y_true = F.one_hot(y_true, num_classes)  # N, H*W -> N,H*W, C
-            y_true = y_true.permute(0, 2, 1)  # H, C, H*W
+            y_pred = rearrange(y_pred, "N C ... -> N C (...)")
+            y_true = F.one_hot(y_true, num_classes)  # (N, *) -> (N, *, C)
+            y_true = rearrange(y_true, "N ... C -> N C (...)")
         else:
             AssertionError("Not implemented")
 
         scores = soft_dice_score(y_pred, y_true.type_as(y_pred), dims=dims)
         if self.log_loss:
-            loss = -torch.log(scores.clamp_min(self.eps))
+            loss = -torch.log(scores.clamp_min(1e-7))
         else:
             loss = 1.0 - scores
         return loss.mean()
@@ -92,8 +91,8 @@ class DiceLoss(nn.Module):
 def soft_dice_score(pred, target, dims=None):
     """
     Shape:
-        - Input: :math:`(N, C, *)` where :math:`*` means any number of additional dimensions
-        - Target: :math:`(N, C, *)`, same shape as the input
+        - Input: :math:`(N, C, *)`
+        - Target: :math:`(N, C, *)`
         - Output: scalar.
     """
     assert pred.size() == target.size()
@@ -105,23 +104,30 @@ def soft_dice_score(pred, target, dims=None):
 
 
 if __name__ == "__main__":
+
+    print("multiclass")
     pred = torch.tensor([[
-        [1., 2., 3.],
-        [2., 6., 4.],
-        [9., 6., 3.],
-        [4., 8., 12.]
-    ]]).permute(0, 2, 1)
-    true = torch.tensor([[2, 1, 0, 2]])
-    print(pred.shape, true.shape)
+        [1., 2., 3., 4.],
+        [2., 6., 4., 4.],
+        [9., 6., 3., 4.]
+    ]]).view(1, 3, 4)
+    true = torch.tensor([[2, 1, 0, 2]]).view(1, 4)
+
     dice = DiceLoss("multiclass", from_logits=True)
     loss = dice(pred, true)
-    print(loss, '== 0.1350?')
+    print(loss.item(), "== 0.5519775748252869")
 
     print("Binary")
     pred = torch.tensor([0.4, 0.2, 0.3, 0.5]).reshape(1, 1, 2, 2)
     true = torch.tensor([0, 1, 0, 1]).reshape(1, 2, 2)
-    print(pred.shape, true.shape)
     dice = DiceLoss("binary", from_logits=True)
     loss = dice(pred, true)
-    print(loss, "== 0.4604?")
+    print(loss.item(), "== 0.46044695377349854")
+
+    print("Binary Logloss")
+    pred = torch.tensor([0.4, 0.2, 0.3, 0.5]).reshape(1, 1, 2, 2)
+    true = torch.tensor([0, 1, 0, 1]).reshape(1, 2, 2)
+    dice = DiceLoss("binary", from_logits=True, log_loss=True)
+    loss = dice(pred, true)
+    print(loss.item(), "== 0.6170141696929932")
 
